import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.LineBorder;
import java.io.*;
import java.net.*;
public class GameClient extends JApplet implements Runnable,InterfaceConstants{
  private boolean myTurn = false;
  private char myToken = ' ';
  private char otherToken = ' ';
  private Cell[][] cell =  new Cell[3][3];
  private JLabel JTitle = new JLabel();
  private JLabel JStatus= new JLabel();
  private int rowSelected;
  private int columnSelected;
  private DataInputStream fromServer;
  private DataOutputStream toServer;
  private boolean continueToPlay = true;
  private boolean waiting = true;
  private boolean isStandAlone = false;
  private String host = "localhost";
  public void init() {
    JPanel PCell = new JPanel();
    PCell.setLayout(new GridLayout(3, 3, 0, 0));
    for (int i = 0; i < 3; i++){
      for (int j = 0; j < 3; j++){
        PCell.add(cell[i][j] = new Cell(i, j));
      }
    }
    PCell.setBorder(new LineBorder(Color.black, 1));
    JTitle.setHorizontalAlignment(JLabel.CENTER);
    JTitle.setFont(new Font("з", Font.BOLD, 18));
    JTitle.setBorder(new LineBorder(Color.BLUE, 1));
    JStatus.setBorder(new LineBorder(Color.BLUE, 1));
    this.getContentPane().add(JTitle, BorderLayout.NORTH);
    this.getContentPane().add(PCell, BorderLayout.CENTER);
    this.getContentPane().add(JStatus, BorderLayout.SOUTH);
    connectToServer();
  }

  private void connectToServer() {
    try {
      Socket socket;
      if (isStandAlone)
        socket = new Socket(host, 9000);
      else
        socket = new Socket(getCodeBase().getHost(), 9000);
      fromServer = new DataInputStream(socket.getInputStream());
      toServer = new DataOutputStream(socket.getOutputStream());
    }
    catch (Exception ex) {
      System.err.println(ex);
    }
    Thread thread = new Thread(this);
    thread.start();
  }

  public void run() {
    try {
      int player = fromServer.readInt();
      if (player == PLAYER1) {
        myToken = 'X';
        otherToken = 'O';
        JTitle.setText("a1,ϥ'X'");
        JStatus.setText("nӤH,ݪa2[J");
        fromServer.readInt();
        JStatus.setText("a2[J,a1U");
        myTurn = true;
      }
      else if (player == PLAYER2) {
        myToken = 'O';
        otherToken = 'X';
        JTitle.setText("a2,ϥ'O'");
        JStatus.setText("C}l,a1");
      }
      while (continueToPlay) {
        if (player == PLAYER1) {
          waitForPlayerAction(); 
          sendMove(); 
          receiveInfoFromServer(); 
        }
        else if (player == PLAYER2) {
          receiveInfoFromServer(); 
          waitForPlayerAction(); 
          sendMove();
        }
      }
    }
    catch (Exception ex) {
    }
  }
  private void waitForPlayerAction() throws InterruptedException {
    while (waiting) {
      Thread.sleep(100);
    }
    waiting = true;
  }
  private void sendMove() throws IOException {
    toServer.writeInt(rowSelected); 
    toServer.writeInt(columnSelected);
  }
  private void receiveInfoFromServer() throws IOException {
    int status = fromServer.readInt();
    if (status == PLAYER1_WON) {
      continueToPlay = false;
      if (myToken == 'X') {
        JStatus.setText("ά,@wĹF!");
      }else if (myToken == 'O') {
        JStatus.setText("U@,AnbڤW!");
        receiveMove();
      }
    }else if (status == PLAYER2_WON) {
      continueToPlay = false;
      if (myToken == 'O') {
        JStatus.setText("C,ĹF(O)!");
      }else if (myToken == 'X') {
        JStatus.setText("F,گu!");
        receiveMove();
      }
    }else if (status == EQUAL) {
      continueToPlay = false;
      JStatus.setText("C,!");
      if (myToken == 'O') {
        receiveMove();
      }
    }else {
      receiveMove();
      JStatus.setText("ڤF");
      myTurn = true; 
    }
  }
  private void receiveMove() throws IOException {
    int row = fromServer.readInt();
    int column = fromServer.readInt();
    cell[row][column].setToken(otherToken);
  }
  private class Cell extends JPanel implements MouseListener {
    private int row;
    private int column;
    private char token = ' ';
    public Cell(int row, int column) {
      this.row = row;
      this.column = column;
      setBorder(new LineBorder(Color.black, 1)); 
      addMouseListener(this);  
    }
    public char getToken() {
      return token;
    }
    public void setToken(char c) {
      token = c;
      repaint();
    }
    protected void paintComponent(Graphics g) {
      super.paintComponent(g);

      if (token == 'X') {
        g.drawLine(10, 10, getWidth() - 10, getHeight() - 10);
        g.drawLine(getWidth() - 10, 10, 10, getHeight() - 10);
      }
      else if (token == 'O') {
        g.drawOval(10, 10, getWidth() - 20, getHeight() - 20);
      }
    }
    public void mouseClicked(MouseEvent e) {
      if ((token == ' ') && myTurn) {
        setToken(myToken); 
        myTurn = false;
        rowSelected = row;
        columnSelected = column;
        JStatus.setText("ݥt@쪱a");
        waiting = false; 
      }
    }
    public void mousePressed(MouseEvent e) {
      // @ java.awt.event.MouseListener k;
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
  }
  public static void main(String[] args) {
    JFrame frame = new JFrame("eCϥΪ̺");
    GameClient applet = new GameClient();
    applet.isStandAlone = true;
    if (args.length == 1) applet.host = args[0];
    frame.getContentPane().add(applet, BorderLayout.CENTER);
    applet.init();
    applet.start();
    frame.setSize(300, 300);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
  }
}


